
#toolbox > expander {
  border-top: 1px solid @unfocused_borders;
  margin-top: 6px;
  padding: 6px 3px 0px 3px;
}

#toolbox > expander:first-child {
  border-top: none;
  margin-top: 0px;
}

#toolbox flowboxchild {
  padding: 0;
}

#diagrams {
  border-top-width: 0px;
}

diagramview {
  background: white;
}

.status-window box {
  padding: 12px;
}